@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2014-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2014      Artem Y. Polyakov <artpol84@gmail.com>.
@@ -52,6 +52,7 @@
 
 #include "src/class/pmix_list.h"
 #include "src/mca/bfrops/bfrops.h"
+#include "src/mca/ptl/base/base.h"
 #include "src/threads/threads.h"
 #include "src/util/argv.h"
 #include "src/util/compress.h"
@@ -60,7 +61,7 @@
 #include "src/util/name_fns.h"
 #include "src/util/output.h"
 #include "src/mca/gds/gds.h"
-#include "src/mca/ptl/ptl.h"
+#include "src/mca/ptl/base/base.h"
 
 #include "pmix_client_ops.h"
 
@@ -88,7 +89,7 @@ PMIX_EXPORT pmix_status_t PMIx_Get(const pmix_proc_t *proc,
                                    const pmix_info_t info[], size_t ninfo,
                                    pmix_value_t **val)
 {
-    pmix_cb_t *cb;
+    pmix_cb_t cb;
     pmix_status_t rc;
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
@@ -104,32 +105,23 @@ PMIX_EXPORT pmix_status_t PMIx_Get(const pmix_proc_t *proc,
                         (NULL == proc) ? "NULL" : PMIX_NAME_PRINT(proc),
                         (NULL == key) ? "NULL" : key);
 
-    /* try to get data directly, without threadshift */
-    if (PMIX_RANK_UNDEF != proc->rank && NULL != key) {
-        if (PMIX_SUCCESS == (rc = _getfn_fastpath(proc, key, info, ninfo, val))) {
-            goto done;
-        }
-    }
-
-    /* create a callback object as we need to pass it to the
-     * recv routine so we know which callback to use when
-     * the return message is recvd */
-    cb = PMIX_NEW(pmix_cb_t);
-    if (PMIX_SUCCESS != (rc = PMIx_Get_nb(proc, key, info, ninfo, _value_cbfunc, cb))) {
-        PMIX_RELEASE(cb);
+    /* create a callback object so we can be notified when
+     * the non-blocking operation is complete */
+    PMIX_CONSTRUCT(&cb, pmix_cb_t);
+    if (PMIX_SUCCESS != (rc = PMIx_Get_nb(proc, key, info, ninfo, _value_cbfunc, &cb))) {
+        PMIX_DESTRUCT(&cb);
         return rc;
     }
 
     /* wait for the data to return */
-    PMIX_WAIT_THREAD(&cb->lock);
-    rc = cb->status;
+    PMIX_WAIT_THREAD(&cb.lock);
+    rc = cb.status;
     if (NULL != val) {
-        *val = cb->value;
-        cb->value = NULL;
+        *val = cb.value;
+        cb.value = NULL;
     }
-    PMIX_RELEASE(cb);
+    PMIX_DESTRUCT(&cb);
 
-  done:
     pmix_output_verbose(2, pmix_client_globals.get_output,
                         "pmix:client get completed");
 
@@ -141,8 +133,15 @@ PMIX_EXPORT pmix_status_t PMIx_Get_nb(const pmix_proc_t *proc, const pmix_key_t
                                       pmix_value_cbfunc_t cbfunc, void *cbdata)
 {
     pmix_cb_t *cb;
-    int rank;
-    char *nm;
+    pmix_status_t rc;
+    size_t n, nfo;
+    bool wantinfo = false;
+    bool haveid = false;
+    pmix_proc_t p;
+    pmix_info_t *iptr;
+    bool copy = false;
+    uint32_t appnum;
+    pmix_value_t *ival = NULL;
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
@@ -180,31 +179,181 @@ PMIX_EXPORT pmix_status_t PMIx_Get_nb(const pmix_proc_t *proc, const pmix_key_t
     /* if the given proc param is NULL, or the nspace is
      * empty, then the caller is referencing our own nspace */
     if (NULL == proc || 0 == strlen(proc->nspace)) {
-        nm = pmix_globals.myid.nspace;
+        PMIX_LOAD_NSPACE(p.nspace, pmix_globals.myid.nspace);
     } else {
-        nm = (char*)proc->nspace;
+        PMIX_LOAD_NSPACE(p.nspace, proc->nspace);
     }
 
     /* if the proc param is NULL, then we are seeking a key that
      * must be globally unique, so communicate this to the hash
      * functions with the UNDEF rank */
     if (NULL == proc) {
-        rank = PMIX_RANK_UNDEF;
+        p.rank = PMIX_RANK_UNDEF;
     } else {
-        rank = proc->rank;
+        p.rank = proc->rank;
     }
+    iptr = (pmix_info_t*)info;
+    nfo = ninfo;
 
     pmix_output_verbose(2, pmix_client_globals.get_output,
-                        "pmix: get_nb value for proc %s:%u key %s",
-                        nm, rank, (NULL == key) ? "NULL" : key);
+                        "pmix: get_nb value for proc %s key %s",
+                        PMIX_NAME_PRINT(&p), (NULL == key) ? "NULL" : key);
 
+    if (!PMIX_PEER_IS_EARLIER(pmix_client_globals.myserver, 3, 1, 5)) {
+        /* ]don't consider the fastpath option
+         * for undefined rank or NULL keys */
+        if (PMIX_RANK_UNDEF == p.rank || NULL == key) {
+            goto doget;
+        }
+        /* if they are asking about a node-level piece of info,
+         * then the rank must be UNDEF */
+        if (pmix_check_node_info(key)) {
+            p.rank = PMIX_RANK_UNDEF;
+            /* the key is node-related - see if the target node is in the
+             * info array and if they tagged the request accordingly */
+            if (NULL != info) {
+                for (n=0; n < ninfo; n++) {
+                    if (PMIX_CHECK_KEY(&info[n], PMIX_NODE_INFO)) {
+                        wantinfo = true;
+                    } else if (PMIX_CHECK_KEY(&info[n], PMIX_HOSTNAME)) {
+                        haveid = true;
+                    } else if (PMIX_CHECK_KEY(&info[n], PMIX_NODEID)) {
+                        haveid = true;
+                    }
+                }
+            }
+            if (wantinfo && haveid) {
+                goto doget;
+            } else if (wantinfo) {
+                /* missing the nodeid/hostname - add our hostname */
+                nfo = ninfo + 1;
+                PMIX_INFO_CREATE(iptr, nfo);
+                for (n=0; n < ninfo; n++) {
+                    PMIX_INFO_XFER(&iptr[n], &info[n]);
+                }
+                PMIX_INFO_LOAD(&iptr[ninfo], PMIX_HOSTNAME, pmix_globals.hostname, PMIX_STRING);
+                copy = true;
+                goto doget;
+            } else if (haveid) {
+                /* flag that we want node info */
+                nfo = ninfo + 1;
+                PMIX_INFO_CREATE(iptr, nfo);
+                for (n=0; n < ninfo; n++) {
+                    PMIX_INFO_XFER(&iptr[n], &info[n]);
+                }
+                PMIX_INFO_LOAD(&iptr[ninfo], PMIX_NODE_INFO, NULL, PMIX_BOOL);
+                copy = true;
+                goto doget;
+            } else {
+                /* missing both */
+                nfo = ninfo + 2;
+                PMIX_INFO_CREATE(iptr, nfo);
+                for (n=0; n < ninfo; n++) {
+                    PMIX_INFO_XFER(&iptr[n], &info[n]);
+                }
+                PMIX_INFO_LOAD(&iptr[ninfo], PMIX_NODE_INFO, NULL, PMIX_BOOL);
+                PMIX_INFO_LOAD(&iptr[ninfo+1], PMIX_HOSTNAME, pmix_globals.hostname, PMIX_STRING);
+                copy = true;
+                goto doget;
+            }
+        }
+
+        /* see if they are asking about an app-level piece of info */
+        wantinfo = false;
+        haveid = false;
+        if (pmix_check_app_info(key)) {
+            p.rank = PMIX_RANK_UNDEF;
+            /* the key is app-related - see if the target appnum is in the
+             * info array and if they tagged the request accordingly */
+            if (NULL != info) {
+                for (n=0; n < ninfo; n++) {
+                    if (PMIX_CHECK_KEY(&info[n], PMIX_APP_INFO)) {
+                        wantinfo = true;
+                    } else if (PMIX_CHECK_KEY(&info[n], PMIX_APPNUM) &&
+                               0 != info[n].value.data.uint32) {
+                        haveid = true;
+                    }
+                }
+            }
+            if (wantinfo && haveid) {
+                goto doget;
+            } else if (wantinfo) {
+                /* missing the appnum - add ours */
+                nfo = ninfo + 1;
+                PMIX_INFO_CREATE(iptr, nfo);
+                for (n=0; n < ninfo; n++) {
+                    PMIX_INFO_XFER(&iptr[n], &info[n]);
+                }
+                /* try to retrieve it */
+                rc = _getfn_fastpath(&pmix_globals.myid, PMIX_APPNUM, NULL, 0, &ival);
+                if (PMIX_SUCCESS != rc) {
+                    appnum = ival->data.uint32;
+                    PMIX_VALUE_RELEASE(ival);
+                } else {
+                    appnum = 0;
+                }
+                PMIX_INFO_LOAD(&iptr[ninfo], PMIX_APPNUM, &appnum, PMIX_UINT32);
+                copy = true;
+                goto doget;
+            } else if (haveid) {
+                /* flag that we want app info */
+                nfo = ninfo + 1;
+                PMIX_INFO_CREATE(iptr, nfo);
+                for (n=0; n < ninfo; n++) {
+                    PMIX_INFO_XFER(&iptr[n], &info[n]);
+                }
+                PMIX_INFO_LOAD(&iptr[ninfo], PMIX_APP_INFO, NULL, PMIX_BOOL);
+                copy = true;
+                goto doget;
+            } else {
+                /* missing both */
+                nfo = ninfo + 2;
+                PMIX_INFO_CREATE(iptr, nfo);
+                for (n=0; n < ninfo; n++) {
+                    PMIX_INFO_XFER(&iptr[n], &info[n]);
+                }
+                PMIX_INFO_LOAD(&iptr[ninfo], PMIX_APP_INFO, NULL, PMIX_BOOL);
+                /* try to retrieve it */
+                rc = _getfn_fastpath(&pmix_globals.myid, PMIX_APPNUM, NULL, 0, &ival);
+                if (PMIX_SUCCESS != rc) {
+                    appnum = ival->data.uint32;
+                    PMIX_VALUE_RELEASE(ival);
+                } else {
+                    appnum = 0;
+                }
+                PMIX_INFO_LOAD(&iptr[ninfo], PMIX_APPNUM, &appnum, PMIX_UINT32);
+                copy = true;
+                goto doget;
+            }
+        }
+
+        /* see if they are requesting session info or requesting cache refresh */
+        for (n=0; n < ninfo; n++) {
+            if (PMIX_CHECK_KEY(info, PMIX_SESSION_INFO)) {
+                goto doget;
+            }
+        }
+    }
+
+    /* try to get data directly, without threadshift */
+    if (PMIX_SUCCESS == (rc = _getfn_fastpath(&p, key, iptr, nfo, &ival))) {
+        if (NULL != cbfunc) {
+            cbfunc(rc, ival, cbdata);
+            /* ownership of the memory in ival is passed to the
+             * user in the cbfunc, so don't release it here */
+        }
+        return rc;
+    }
+
+  doget:
     /* threadshift this request so we can access global structures */
     cb = PMIX_NEW(pmix_cb_t);
-    cb->pname.nspace = strdup(nm);
-    cb->pname.rank = rank;
+    cb->pname.nspace = strdup(p.nspace);
+    cb->pname.rank = p.rank;
     cb->key = (char*)key;
-    cb->info = (pmix_info_t*)info;
-    cb->ninfo = ninfo;
+    cb->info = iptr;
+    cb->ninfo = nfo;
+    cb->infocopy = copy;
     cb->cbfunc.valuefn = cbfunc;
     cb->cbdata = cbdata;
     PMIX_THREADSHIFT(cb, _getnbfn);
@@ -298,6 +447,7 @@ static void _getnb_cbfunc(struct pmix_peer_t *pr,
     int32_t cnt;
     pmix_proc_t proc;
     pmix_kval_t *kv;
+    bool diffnspace;
 
     pmix_output_verbose(2, pmix_client_globals.get_output,
                         "pmix: get_nb callback recvd");
@@ -312,6 +462,9 @@ static void _getnb_cbfunc(struct pmix_peer_t *pr,
     pmix_strncpy(proc.nspace, cb->pname.nspace, PMIX_MAX_NSLEN);
     proc.rank = cb->pname.rank;
 
+    /* check for a different nspace */
+    diffnspace = !PMIX_CHECK_NSPACE(pmix_globals.myid.nspace, proc.nspace);
+
     /* a zero-byte buffer indicates that this recv is being
      * completed due to a lost connection */
     if (PMIX_BUFFER_IS_EMPTY(buf)) {
@@ -331,10 +484,9 @@ static void _getnb_cbfunc(struct pmix_peer_t *pr,
     }
 
     if (PMIX_SUCCESS != ret) {
-        PMIX_ERROR_LOG(ret);
         goto done;
     }
-    if (PMIX_RANK_UNDEF == proc.rank) {
+    if (PMIX_RANK_UNDEF == proc.rank || diffnspace) {
         PMIX_GDS_ACCEPT_KVS_RESP(rc, pmix_globals.mypeer, buf);
     } else {
         PMIX_GDS_ACCEPT_KVS_RESP(rc, pmix_client_globals.myserver, buf);
@@ -357,9 +509,18 @@ static void _getnb_cbfunc(struct pmix_peer_t *pr,
             /* fetch the data from server peer module - since it is passing
              * it back to the user, we need a copy of it */
             cb->copy = true;
-            if (PMIX_RANK_UNDEF == proc.rank) {
+            if (PMIX_RANK_UNDEF == proc.rank || diffnspace) {
+                if (PMIX_PEER_IS_EARLIER(pmix_client_globals.myserver, 3, 1, 5)) {
+                    /* everything is under rank=wildcard */
+                    proc.rank = PMIX_RANK_WILDCARD;
+                }
                 PMIX_GDS_FETCH_KV(rc, pmix_globals.mypeer, cb);
             } else {
+                if (PMIX_RANK_UNDEF == proc.rank &&
+                    PMIX_PEER_IS_EARLIER(pmix_client_globals.myserver, 3, 1, 5)) {
+                    /* everything is under rank=wildcard */
+                    proc.rank = PMIX_RANK_WILDCARD;
+                }
                 PMIX_GDS_FETCH_KV(rc, pmix_client_globals.myserver, cb);
             }
             if (PMIX_SUCCESS == rc) {
@@ -409,7 +570,7 @@ static pmix_status_t process_values(pmix_value_t **v, pmix_cb_t *cb)
     }
     /* we will return the data as an array of pmix_info_t
      * in the kvs pmix_value_t */
-    val = (pmix_value_t*)malloc(sizeof(pmix_value_t));
+    PMIX_VALUE_CREATE(val, 1);
     if (NULL == val) {
         return PMIX_ERR_NOMEM;
     }
@@ -562,6 +723,7 @@ static void _getnbfn(int fd, short flags, void *cbdata)
     pmix_proc_t proc;
     bool optional = false;
     bool immediate = false;
+    bool internal_only = false;
     struct timeval tv;
     pmix_query_caddy_t *cd;
 
@@ -597,14 +759,10 @@ static void _getnbfn(int fd, short flags, void *cbdata)
                 }
             } else if (PMIX_CHECK_KEY(&cb->info[n], PMIX_DATA_SCOPE)) {
                 cb->scope = cb->info[n].value.data.scope;
-            } else if (PMIX_CHECK_KEY(&cb->info[n], PMIX_SESSION_INFO)) {
-                cb->level = PMIX_LEVEL_SESSION;
-            } else if (PMIX_CHECK_KEY(&cb->info[n], PMIX_JOB_INFO)) {
-                cb->level = PMIX_LEVEL_JOB;
-            } else if (PMIX_CHECK_KEY(&cb->info[n], PMIX_APP_INFO)) {
-                cb->level = PMIX_LEVEL_APP;
-            } else if (PMIX_CHECK_KEY(&cb->info[n], PMIX_NODE_INFO)) {
-                cb->level = PMIX_LEVEL_NODE;
+            } else if (PMIX_CHECK_KEY(&cb->info[n], PMIX_NODE_INFO) ||
+                       PMIX_CHECK_KEY(&cb->info[n], PMIX_APP_INFO) ||
+                       PMIX_CHECK_KEY(&cb->info[n], PMIX_SESSION_INFO)) {
+                internal_only = true;
             }
         }
     }
@@ -624,7 +782,7 @@ static void _getnbfn(int fd, short flags, void *cbdata)
 
     /* if the key is NULL or starts with "pmix", then they are looking
      * for data that was provided by the server at startup */
-    if (NULL == cb->key || 0 == strncmp(cb->key, "pmix", 4)) {
+    if (!internal_only && (NULL == cb->key || 0 == strncmp(cb->key, "pmix", 4))) {
         cb->proc = &proc;
         /* fetch the data from my server's module - since we are passing
          * it back to the user, we need a copy of it */
@@ -639,11 +797,13 @@ static void _getnbfn(int fd, short flags, void *cbdata)
         if (PMIX_SUCCESS != rc) {
             pmix_output_verbose(5, pmix_client_globals.get_output,
                                 "pmix:client job-level data NOT found");
-            if (0 != strncmp(cb->pname.nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN)) {
+            if (!PMIX_CHECK_NSPACE(cb->pname.nspace, pmix_globals.myid.nspace)) {
                 /* we are asking about the job-level info from another
                  * namespace. It seems that we don't have it - go and
-                 * ask server
+                 * ask server and indicate we only need job-level info
+                 * by setting the rank to WILDCARD
                  */
+                proc.rank = PMIX_RANK_WILDCARD;
                 goto request;
             } else if (NULL != cb->key) {
                 /* if immediate was given, then we are being directed to
@@ -724,8 +884,8 @@ static void _getnbfn(int fd, short flags, void *cbdata)
     /* if we got here, then we don't have the data for this proc. If we
      * are a server, or we are a client and not connected, then there is
      * nothing more we can do */
-    if (PMIX_PROC_IS_SERVER(pmix_globals.mypeer) ||
-        (!PMIX_PROC_IS_SERVER(pmix_globals.mypeer) && !pmix_globals.connected)) {
+    if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer) ||
+        (!PMIX_PEER_IS_SERVER(pmix_globals.mypeer) && !pmix_globals.connected)) {
         rc = PMIX_ERR_NOT_FOUND;
         goto respond;
     }
@@ -745,8 +905,7 @@ static void _getnbfn(int fd, short flags, void *cbdata)
      * this nspace:rank. If we do, then no need to ask again as the
      * request will return _all_ data from that proc */
     PMIX_LIST_FOREACH(cbret, &pmix_client_globals.pending_requests, pmix_cb_t) {
-        if (0 == strncmp(cbret->pname.nspace, cb->pname.nspace, PMIX_MAX_NSLEN) &&
-            cbret->pname.rank == cb->pname.rank) {
+        if (PMIX_CHECK_PROCID(&cbret->pname, &cb->pname)) {
             /* we do have a pending request, but we still need to track this
              * outstanding request so we can satisfy it once the data is returned */
             pmix_list_append(&pmix_client_globals.pending_requests, &cb->super);
@@ -756,16 +915,16 @@ static void _getnbfn(int fd, short flags, void *cbdata)
 
     /* we don't have a pending request, so let's create one - don't worry
      * about packing the key as we return everything from that proc */
-    msg = _pack_get(cb->pname.nspace, cb->pname.rank, cb->info, cb->ninfo, PMIX_GETNB_CMD);
+    msg = _pack_get(cb->pname.nspace, proc.rank, cb->info, cb->ninfo, PMIX_GETNB_CMD);
     if (NULL == msg) {
         rc = PMIX_ERROR;
         goto respond;
     }
 
     pmix_output_verbose(2, pmix_client_globals.get_output,
-                        "%s REQUESTING DATA FROM SERVER FOR %s KEY %s",
+                        "%s REQUESTING DATA FROM SERVER FOR %s:%s KEY %s",
                         PMIX_NAME_PRINT(&pmix_globals.myid),
-                        PMIX_NAME_PRINT(cb->proc), cb->key);
+                        cb->proc->nspace, PMIX_RANK_PRINT(proc.rank), cb->key);
 
     /* track the callback object */
     pmix_list_append(&pmix_client_globals.pending_requests, &cb->super);
