@@ -205,6 +205,12 @@ PMIX_EXPORT extern pmix_bfrops_globals_t pmix_bfrops_globals;
         free(tmpbuf);                                                       \
     } while (0)
 
+/* for backwards compatibility */
+typedef struct pmix_info_array {
+    size_t size;
+    pmix_info_t *array;
+} pmix_info_array_t;
+
 
 /**
  * Internal struct used for holding registered bfrop functions
@@ -268,7 +274,7 @@ PMIX_EXPORT pmix_status_t pmix_bfrops_stub_copy_payload(struct pmix_peer_t *peer
                                                         pmix_buffer_t *src);
 PMIX_EXPORT pmix_status_t pmix_bfrops_stub_value_xfer(struct pmix_peer_t *peer,
                                                       pmix_value_t *dest,
-                                                      pmix_value_t *src);
+                                                      const pmix_value_t *src);
 PMIX_EXPORT void pmix_bfrops_stub_value_load(struct pmix_peer_t *peer,
                                              pmix_value_t *v, void *data,
                                              pmix_data_type_t type);
@@ -496,9 +502,6 @@ PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_string(char **dest, char *src,
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_value(pmix_value_t **dest,
                                                       pmix_value_t *src,
                                                       pmix_data_type_t type);
-PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_array(pmix_info_array_t **dest,
-                                                      pmix_info_array_t *src,
-                                                      pmix_data_type_t type);
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_proc(pmix_proc_t **dest,
                                                      pmix_proc_t *src,
                                                      pmix_data_type_t type);
@@ -514,9 +517,6 @@ PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_buf(pmix_buffer_t **dest,
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_kval(pmix_kval_t **dest,
                                                      pmix_kval_t *src,
                                                      pmix_data_type_t type);
-PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_modex(pmix_modex_data_t **dest,
-                                                      pmix_modex_data_t *src,
-                                                      pmix_data_type_t type);
 PMIX_EXPORT pmix_status_t pmix_bfrop_base_copy_persist(pmix_persistence_t **dest,
                                                        pmix_persistence_t *src,
                                                        pmix_data_type_t type);
@@ -538,10 +538,6 @@ PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_query(pmix_query_t **dest,
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_envar(pmix_envar_t **dest,
                                                       pmix_envar_t *src,
                                                       pmix_data_type_t type);
-/**** DEPRECATED ****/
-PMIX_EXPORT pmix_status_t pmix_bfrops_base_copy_array(pmix_info_array_t **dest,
-                                                      pmix_info_array_t *src,
-                                                      pmix_data_type_t type);
 
 /*
 * "Standard" print functions
@@ -596,8 +592,6 @@ PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_status(char **output, char *pre
 
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_value(char **output, char *prefix,
                                                        pmix_value_t *src, pmix_data_type_t type);
-PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_array(char **output, char *prefix,
-                                                       pmix_info_array_t *src, pmix_data_type_t type);
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_proc(char **output, char *prefix,
                                                       pmix_proc_t *src, pmix_data_type_t type);
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_app(char **output, char *prefix,
@@ -608,8 +602,6 @@ PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_buf(char **output, char *prefix
                                                      pmix_buffer_t *src, pmix_data_type_t type);
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_kval(char **output, char *prefix,
                                                       pmix_kval_t *src, pmix_data_type_t type);
-PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_modex(char **output, char *prefix,
-                                                       pmix_modex_data_t *src, pmix_data_type_t type);
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_persist(char **output, char *prefix,
                                                          pmix_persistence_t *src, pmix_data_type_t type);
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_print_bo(char **output, char *prefix,
@@ -678,7 +670,7 @@ PMIX_EXPORT pmix_status_t pmix_bfrops_base_value_unload(pmix_value_t *kv,
                                                         size_t *sz);
 
 PMIX_EXPORT pmix_status_t pmix_bfrops_base_value_xfer(pmix_value_t *p,
-                                                      pmix_value_t *src);
+                                                      const pmix_value_t *src);
 
 PMIX_EXPORT pmix_value_cmp_t pmix_bfrops_base_value_cmp(pmix_value_t *p,
                                                         pmix_value_t *p1);
