@@ -55,13 +55,6 @@ static void _notify_complete(pmix_status_t status, void *cbdata)
     PMIX_RELEASE(chain);
 }
 
-static void _timeout(int sd, short args, void *cbdata)
-{
-    pmix_server_trkr_t *trk = (pmix_server_trkr_t*)cbdata;
-
-    PMIX_RELEASE(trk);
-}
-
 static void lcfn(pmix_status_t status, void *cbdata)
 {
     pmix_peer_t *peer = (pmix_peer_t*)cbdata;
@@ -76,7 +69,6 @@ void pmix_ptl_base_lost_connection(pmix_peer_t *peer, pmix_status_t err)
     pmix_ptl_posted_recv_t *rcv;
     pmix_buffer_t buf;
     pmix_ptl_hdr_t hdr;
-    struct timeval tv = {1200, 0};
     pmix_proc_t proc;
     pmix_status_t rc;
 
@@ -95,8 +87,8 @@ void pmix_ptl_base_lost_connection(pmix_peer_t *peer, pmix_status_t err)
     }
     CLOSE_THE_SOCKET(peer->sd);
 
-    if (PMIX_PROC_IS_SERVER(pmix_globals.mypeer) &&
-        !PMIX_PROC_IS_TOOL(pmix_globals.mypeer)) {
+    if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer) &&
+        !PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
         /* if I am a server, then we need to ensure that
          * we properly account for the loss of this client
          * from any local collectives in which it was
@@ -114,59 +106,60 @@ void pmix_ptl_base_lost_connection(pmix_peer_t *peer, pmix_status_t err)
                 /* remove it from the list */
                 pmix_list_remove_item(&trk->local_cbs, &rinfo->super);
                 PMIX_RELEASE(rinfo);
-                trk->lost_connection = true;  // mark that a peer's connection was lost
-                if (0 == pmix_list_get_size(&trk->local_cbs)) {
-                    /* this tracker is complete, so release it - there
-                     * is nobody waiting for a response */
-                    pmix_list_remove_item(&pmix_server_globals.collectives, &trk->super);
-                    /* do NOT release the tracker here as the host may
-                     * have a copy they will return later. However, they
-                     * might never call back, so set a LONG timeout to
-                     * we avoid a memory leak if they don't */
-                    pmix_event_evtimer_set(pmix_globals.evbase, &trk->ev,
-                                           _timeout, trk);
-                    pmix_event_evtimer_add(&trk->ev, &tv);
-                    trk->event_active = true;
-                    break;
+                /* if the host has already been called for this tracker,
+                 * then do nothing here - just wait for the host to return
+                 * from the operation */
+                if (trk->host_called) {
+                    continue;
                 }
-                /* if there are other participants waiting for a response,
-                 * we need to let them know that this proc has disappeared
-                 * as otherwise the collective will never complete */
-                if (PMIX_FENCENB_CMD == trk->type) {
-                    if (NULL != trk->modexcbfunc) {
-                        /* do NOT release the tracker here as the host may
-                         * have a copy they will return later. However, they
-                         * might never call back, so set a LONG timeout to
-                         * we avoid a memory leak if they don't */
-                        pmix_event_evtimer_set(pmix_globals.evbase, &trk->ev,
-                                               _timeout, trk);
-                        pmix_event_evtimer_add(&trk->ev, &tv);
-                        trk->event_active = true;
-                        trk->modexcbfunc(PMIX_ERR_LOST_CONNECTION_TO_CLIENT, NULL, 0, trk, NULL, NULL);
-                    }
-                } else if (PMIX_CONNECTNB_CMD == trk->type) {
-                    if (NULL != trk->op_cbfunc) {
-                        /* do NOT release the tracker here as the host may
-                         * have a copy they will return later. However, they
-                         * might never call back, so set a LONG timeout to
-                         * we avoid a memory leak if they don't */
-                        pmix_event_evtimer_set(pmix_globals.evbase, &trk->ev,
-                                               _timeout, trk);
-                        pmix_event_evtimer_add(&trk->ev, &tv);
-                        trk->event_active = true;
-                        trk->op_cbfunc(PMIX_ERR_LOST_CONNECTION_TO_CLIENT, trk);
-                    }
-                } else if (PMIX_DISCONNECTNB_CMD == trk->type) {
-                    if (NULL != trk->op_cbfunc) {
-                        /* do NOT release the tracker here as the host may
-                         * have a copy they will return later. However, they
-                         * might never call back, so set a LONG timeout to
-                         * we avoid a memory leak if they don't */
-                        pmix_event_evtimer_set(pmix_globals.evbase, &trk->ev,
-                                               _timeout, trk);
-                        pmix_event_evtimer_add(&trk->ev, &tv);
-                        trk->event_active = true;
-                        trk->op_cbfunc(PMIX_ERR_LOST_CONNECTION_TO_CLIENT, trk);
+                if (trk->def_complete && trk->nlocal == pmix_list_get_size(&trk->local_cbs)) {
+                    /* if this is a local-only collective, then resolve it now */
+                    if (trk->local) {
+                        /* everyone else has called in - we need to let them know
+                         * that this proc has disappeared
+                         * as otherwise the collective will never complete */
+                        if (PMIX_FENCENB_CMD == trk->type) {
+                            if (NULL != trk->modexcbfunc) {
+                                trk->modexcbfunc(PMIX_ERR_LOST_CONNECTION_TO_CLIENT, NULL, 0, trk, NULL, NULL);
+                            }
+                        } else if (PMIX_CONNECTNB_CMD == trk->type) {
+                            if (NULL != trk->op_cbfunc) {
+                                trk->op_cbfunc(PMIX_ERR_LOST_CONNECTION_TO_CLIENT, trk);
+                            }
+                        } else if (PMIX_DISCONNECTNB_CMD == trk->type) {
+                            if (NULL != trk->op_cbfunc) {
+                                trk->op_cbfunc(PMIX_ERR_LOST_CONNECTION_TO_CLIENT, trk);
+                            }
+                        }
+                    } else {
+                        /* if the host has not been called, then we need to see if
+                         * the collective is locally complete without this lost
+                         * participant. If so, then we need to pass the call
+                         * up to the host as otherwise the global collective will hang */
+                        if (PMIX_FENCENB_CMD == trk->type) {
+                            trk->host_called = true;
+                            rc = pmix_host_server.fence_nb(trk->pcs, trk->npcs,
+                                                           trk->info, trk->ninfo,
+                                                           NULL, 0, trk->modexcbfunc, trk);
+                            if (PMIX_SUCCESS != rc) {
+                                pmix_list_remove_item(&pmix_server_globals.collectives, &trk->super);
+                                PMIX_RELEASE(trk);
+                            }
+                        } else if (PMIX_CONNECTNB_CMD == trk->type) {
+                            trk->host_called = true;
+                            rc = pmix_host_server.connect(trk->pcs, trk->npcs, trk->info, trk->ninfo, trk->op_cbfunc, trk);
+                            if (PMIX_SUCCESS != rc) {
+                                pmix_list_remove_item(&pmix_server_globals.collectives, &trk->super);
+                                PMIX_RELEASE(trk);
+                            }
+                        } else if (PMIX_DISCONNECTNB_CMD == trk->type) {
+                            trk->host_called = true;
+                            rc = pmix_host_server.disconnect(trk->pcs, trk->npcs, trk->info, trk->ninfo, trk->op_cbfunc, trk);
+                            if (PMIX_SUCCESS != rc) {
+                                pmix_list_remove_item(&pmix_server_globals.collectives, &trk->super);
+                                PMIX_RELEASE(trk);
+                            }
+                        }
                     }
                 }
             }
@@ -192,7 +185,7 @@ void pmix_ptl_base_lost_connection(pmix_peer_t *peer, pmix_status_t err)
         /* purge any notifications cached for this client */
         pmix_server_purge_events(peer, NULL);
 
-        if (PMIX_PROC_IS_LAUNCHER(pmix_globals.mypeer)) {
+        if (PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer)) {
             /* only connection I can lose is to my server, so mark it */
             pmix_globals.connected = false;
         } else {
@@ -200,7 +193,7 @@ void pmix_ptl_base_lost_connection(pmix_peer_t *peer, pmix_status_t err)
             pmix_psensor.stop(peer, NULL);
         }
 
-        if (!peer->finalized && !PMIX_PROC_IS_TOOL(peer) && !pmix_globals.mypeer->finalized) {
+        if (!peer->finalized && !PMIX_PEER_IS_TOOL(peer) && !pmix_globals.mypeer->finalized) {
             /* if this peer already called finalize, then
              * we are just seeing their connection go away
              * when they terminate - so do not generate
