@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2014-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -125,7 +125,7 @@ PMIX_CLASS_INSTANCE(pmix_output_stream_t, pmix_object_t, construct, destruct);
 bool pmix_output_init(void)
 {
     int i;
-    char hostname[PMIX_MAXHOSTNAMELEN];
+    char hostname[PMIX_MAXHOSTNAMELEN] = {0};
     char *str;
 
     if (initialized) {
@@ -176,7 +176,7 @@ bool pmix_output_init(void)
     } else {
         verbose.lds_want_stderr = true;
     }
-    gethostname(hostname, sizeof(hostname));
+    gethostname(hostname, sizeof(hostname)-1);
     hostname[sizeof(hostname)-1] = '\0';
     if (0 > asprintf(&verbose.lds_prefix, "[%s:%05d] ", hostname, getpid())) {
         return PMIX_ERR_NOMEM;
@@ -256,7 +256,7 @@ bool pmix_output_switch(int output_id, bool enable)
 void pmix_output_reopen_all(void)
 {
     char *str;
-    char hostname[PMIX_MAXHOSTNAMELEN];
+    char hostname[PMIX_MAXHOSTNAMELEN] = {0};
 
     str = getenv("PMIX_OUTPUT_STDERR_FD");
     if (NULL != str) {
@@ -332,15 +332,10 @@ PMIX_EXPORT void pmix_output(int output_id, const char *format, ...)
 /*
  * Send a message to a stream if the verbose level is high enough
  */
- PMIX_EXPORT void pmix_output_verbose(int level, int output_id, const char *format, ...)
+ PMIX_EXPORT bool pmix_output_check_verbosity(int level, int output_id)
 {
-    if (output_id >= 0 && output_id < PMIX_OUTPUT_MAX_STREAMS &&
-        info[output_id].ldi_verbose_level >= level) {
-        va_list arglist;
-        va_start(arglist, format);
-        output(output_id, format, arglist);
-        va_end(arglist);
-    }
+    return (output_id >= 0 && output_id < PMIX_OUTPUT_MAX_STREAMS &&
+        info[output_id].ldi_verbose_level >= level);
 }
 
 
