@@ -12,6 +12,7 @@
  *
  */
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -21,6 +22,7 @@
 
 #include "pmix_server.h"
 #include "src/include/pmix_globals.h"
+#include "src/util/error.h"
 
 #include "test_server.h"
 #include "test_common.h"
@@ -28,6 +30,7 @@
 #include "server_callbacks.h"
 
 int my_server_id = 0;
+int test_fail = 0;
 
 server_info_t *my_server_info = NULL;
 pmix_list_t *server_list = NULL;
@@ -41,10 +44,14 @@ static void sdes(server_info_t *s)
         event_del(s->evread);
     }
     s->evread = NULL;
+    if (NULL != s->hostname) {
+        free(s->hostname);
+    }
 }
 
 static void scon(server_info_t *s)
 {
+    s->hostname = NULL;
     s->idx = 0;
     s->pid = 0;
     s->rd_fd = -1;
@@ -115,14 +122,29 @@ static void fill_seq_ranks_array(size_t nprocs, int base_rank, char **ranks)
     }
 }
 
+static int server_find_id(const char *nspace, int rank)
+{
+    server_nspace_t *tmp;
+
+    PMIX_LIST_FOREACH(tmp, server_nspace, server_nspace_t) {
+        if (0 == strcmp(tmp->name, nspace)) {
+            return tmp->task_map[rank];
+        }
+    }
+    return -1;
+}
+
 static void set_namespace(int local_size, int univ_size,
                           int base_rank, char *name)
 {
     size_t ninfo;
     pmix_info_t *info;
     ninfo = 8;
-    char *regex, *ppn;
-    char *ranks = NULL;
+    char *regex, *ppn, *tmp;
+    char *ranks = NULL, **nodes = NULL;
+    char **rks=NULL;
+    int i;
+    int rc;
 
     PMIX_INFO_CREATE(info, ninfo);
     pmix_strncpy(info[0].key, PMIX_UNIV_SIZE, PMIX_MAX_KEYLEN);
@@ -145,23 +167,62 @@ static void set_namespace(int local_size, int univ_size,
     pmix_strncpy(info[3].key, PMIX_LOCAL_PEERS, PMIX_MAX_KEYLEN);
     info[3].value.type = PMIX_STRING;
     info[3].value.data.string = strdup(ranks);
-    free(ranks);
 
-    PMIx_generate_regex(NODE_NAME, &regex);
-    pmix_strncpy(info[4].key, PMIX_NODE_MAP, PMIX_MAX_KEYLEN);
-    info[4].value.type = PMIX_STRING;
-    info[4].value.data.string = strdup(regex);
+    /* assemble the node and proc map info */
+    if (1 == params.nservers) {
+        pmix_argv_append_nosize(&nodes, my_server_info->hostname);
+    } else {
+        char hostname[PMIX_MAXHOSTNAMELEN];
+        for (i = 0; i < params.nservers; i++) {
+            snprintf(hostname, PMIX_MAXHOSTNAMELEN, "node%d", i);
+            pmix_argv_append_nosize(&nodes, hostname);
+        }
+    }
 
-    /* generate the global proc map */
-    fill_seq_ranks_array(univ_size, 0, &ranks);
-    if (NULL == ranks) {
-        return;
+    if (NULL != nodes) {
+        tmp = pmix_argv_join(nodes, ',');
+        pmix_argv_free(nodes);
+        nodes = NULL;
+        if (PMIX_SUCCESS != (rc = PMIx_generate_regex(tmp, &regex) )) {
+            PMIX_ERROR_LOG(rc);
+            return;
+        }
+        free(tmp);
+        PMIX_INFO_LOAD(&info[4], PMIX_NODE_MAP, regex, PMIX_STRING);
+    }
+
+    /* generate the global proc map - if we have two
+     * servers, then the procs not on this server must
+     * be on the other */
+    if (2 == params.nservers) {
+        pmix_argv_append_nosize(&rks, ranks);
+        free(ranks);
+        nodes = NULL;
+        if (0 == my_server_id) {
+            for (i=base_rank+local_size; i < univ_size; i++) {
+                asprintf(&ppn, "%d", i);
+                pmix_argv_append_nosize(&nodes, ppn);
+                free(ppn);
+            }
+            ppn = pmix_argv_join(nodes, ',');
+            pmix_argv_append_nosize(&rks, ppn);
+            free(ppn);
+        } else {
+            for (i=0; i < base_rank; i++) {
+                asprintf(&ppn, "%d", i);
+                pmix_argv_append_nosize(&nodes, ppn);
+                free(ppn);
+            }
+            ppn = pmix_argv_join(nodes, ',');
+            pmix_argv_prepend_nosize(&rks, ppn);
+            free(ppn);
+        }
+        ranks = pmix_argv_join(rks, ';');
     }
     PMIx_generate_ppn(ranks, &ppn);
     free(ranks);
-    pmix_strncpy(info[5].key, PMIX_PROC_MAP, PMIX_MAX_KEYLEN);
-    info[5].value.type = PMIX_STRING;
-    info[5].value.data.string = strdup(ppn);
+    PMIX_INFO_LOAD(&info[5], PMIX_PROC_MAP, ppn, PMIX_STRING);
+    free(ppn);
 
     pmix_strncpy(info[6].key, PMIX_JOB_SIZE, PMIX_MAX_KEYLEN);
     info[6].value.type = PMIX_UINT32;
@@ -171,7 +232,7 @@ static void set_namespace(int local_size, int univ_size,
     info[7].value.type = PMIX_UINT32;
     info[7].value.data.uint32 = getpid ();
 
-    int in_progress = 1, rc;
+    int in_progress = 1;
     if (PMIX_SUCCESS == (rc = PMIx_server_register_nspace(name, local_size,
                                     info, ninfo, release_cb, &in_progress))) {
         PMIX_WAIT_FOR_COMPLETION(in_progress);
@@ -187,7 +248,7 @@ static void server_unpack_procs(char *buf, size_t size)
     char *nspace;
 
     while ((size_t)(ptr - buf) < size) {
-        ns_count = *(size_t *)ptr;
+        memcpy (&ns_count, ptr, sizeof(size_t));
         ptr += sizeof(size_t);
 
         for (i = 0; i < ns_count; i++) {
@@ -195,16 +256,16 @@ static void server_unpack_procs(char *buf, size_t size)
             size_t ltasks, ntasks;
             int server_id;
 
-            server_id = *(int *)ptr;
+            memcpy (&server_id, ptr, sizeof(int));
             ptr += sizeof(int);
 
             nspace = ptr;
             ptr += PMIX_MAX_NSLEN+1;
 
-            ntasks = *(size_t *)ptr;
+            memcpy (&ntasks, ptr, sizeof(size_t));
             ptr += sizeof(size_t);
 
-            ltasks = *(size_t *)ptr;
+            memcpy (&ltasks, ptr, sizeof(size_t));
             ptr += sizeof(size_t);
 
             PMIX_LIST_FOREACH(tmp, server_nspace, server_nspace_t) {
@@ -226,7 +287,8 @@ static void server_unpack_procs(char *buf, size_t size)
             }
             size_t i;
             for (i = 0; i < ltasks; i++) {
-                int rank = *(int *)ptr;
+                int rank;
+                memcpy (&rank, ptr, sizeof(int));
                 ptr += sizeof(int);
                 if (ns_item->task_map[rank] >= 0) {
                     continue;
@@ -614,18 +676,6 @@ int server_fence_contrib(char *data, size_t ndata,
     return rc;
 }
 
-static int server_find_id(const char *nspace, int rank)
-{
-    server_nspace_t *tmp;
-
-    PMIX_LIST_FOREACH(tmp, server_nspace, server_nspace_t) {
-        if (0 == strcmp(tmp->name, nspace)) {
-            return tmp->task_map[rank];
-        }
-    }
-    return -1;
-}
-
 static int server_pack_dmdx(int sender_id, const char *nspace, int rank,
                             char **buf)
 {
@@ -724,9 +774,73 @@ error:
     return PMIX_ERROR;
 }
 
+static void set_handler_default(int sig)
+{
+    struct sigaction act;
+
+    act.sa_handler = SIG_DFL;
+    act.sa_flags = 0;
+    sigemptyset(&act.sa_mask);
+
+    sigaction(sig, &act, (struct sigaction *)0);
+}
+
+static pmix_event_t handler;
+static void wait_signal_callback(int fd, short event, void *arg)
+{
+    pmix_event_t *sig = (pmix_event_t*) arg;
+    int status;
+    pid_t pid;
+    int i;
+
+    if (SIGCHLD != pmix_event_get_signal(sig)) {
+        return;
+    }
+
+    /* we can have multiple children leave but only get one
+     * sigchild callback, so reap all the waitpids until we
+     * don't get anything valid back */
+    while (1) {
+        pid = waitpid(-1, &status, WNOHANG);
+        if (-1 == pid && EINTR == errno) {
+            /* try it again */
+            continue;
+        }
+        /* if we got garbage, then nothing we can do */
+        if (pid <= 0) {
+            goto done;
+        }
+        /* we are already in an event, so it is safe to access the list */
+        for(i=0; i < cli_info_cnt; i++){
+            if( cli_info[i].pid == pid ){
+                /* found it! */
+                if (WIFEXITED(status)) {
+                    cli_info[i].exit_code = WEXITSTATUS(status);
+                } else {
+                    if (WIFSIGNALED(status)) {
+                        cli_info[i].exit_code = WTERMSIG(status) + 128;
+                    }
+                }
+                cli_cleanup(&cli_info[i]);
+                cli_info[i].alive = false;
+                break;
+            }
+        }
+    }
+  done:
+    for(i=0; i < cli_info_cnt; i++){
+        if (cli_info[i].alive) {
+            /* someone is still alive */
+            return;
+        }
+    }
+    /* get here if nobody is still alive */
+    test_complete = true;
+}
+
 int server_init(test_params *params)
 {
-    pmix_info_t info[1];
+    pmix_info_t info[2];
     int rc = PMIX_SUCCESS;
 
     /* fork/init servers procs */
@@ -754,7 +868,9 @@ int server_init(test_params *params)
                 }
                 if (pid == 0) {
                     server_list = PMIX_NEW(pmix_list_t);
+                    my_server_info = server_info;
                     my_server_id = i;
+                    asprintf(&server_info->hostname, "node%d", i);
                     server_info->idx = 0;
                     server_info->pid = getppid();
                     server_info->rd_fd = fd1[0];
@@ -765,6 +881,7 @@ int server_init(test_params *params)
                     pmix_list_append(server_list, &server_info->super);
                     break;
                 }
+                asprintf(&server_info->hostname, "node%d", i);
                 server_info->idx = i;
                 server_info->pid = pid;
                 server_info->wr_fd = fd1[1];
@@ -774,6 +891,7 @@ int server_init(test_params *params)
                 close(fd2[1]);
             } else {
                 my_server_info = server_info;
+                server_info->hostname = strdup("node0");
                 server_info->pid = getpid();
                 server_info->idx  = 0;
                 server_info->rd_fd = fd1[0];
@@ -791,13 +909,13 @@ int server_init(test_params *params)
                 params->nprocs / params->nservers + 1 :
                 params->nprocs / params->nservers;
     /* setup the server library */
-    (void)strncpy(info[0].key, PMIX_SOCKET_MODE, PMIX_MAX_KEYLEN);
-    info[0].value.type = PMIX_UINT32;
-    info[0].value.data.uint32 = 0666;
+    uint32_t u32 = 0666;
+    PMIX_INFO_LOAD(&info[0], PMIX_SOCKET_MODE, &u32, PMIX_UINT32);
+    PMIX_INFO_LOAD(&info[1], PMIX_HOSTNAME, my_server_info->hostname, PMIX_STRING);
 
     server_nspace = PMIX_NEW(pmix_list_t);
 
-    if (PMIX_SUCCESS != (rc = PMIx_server_init(&mymodule, info, 1))) {
+    if (PMIX_SUCCESS != (rc = PMIx_server_init(&mymodule, info, 2))) {
         TEST_ERROR(("Init failed with error %d", rc));
         goto error;
     }
@@ -812,9 +930,17 @@ int server_init(test_params *params)
         }
     }
 
+#if 0
     /* register the errhandler */
     PMIx_Register_event_handler(NULL, 0, NULL, 0,
                                 errhandler, errhandler_reg_callbk, NULL);
+#endif
+
+    /* setup to see sigchld on the forked tests */
+    pmix_event_assign(&handler, pmix_globals.evbase, SIGCHLD,
+                      EV_SIGNAL|EV_PERSIST, wait_signal_callback, &handler);
+    pmix_event_add(&handler, NULL);
+
 
     if (0 != (rc = server_barrier())) {
         goto error;
@@ -832,6 +958,7 @@ int server_finalize(test_params *params)
     int rc = PMIX_SUCCESS;
     int total_ret = 0;
 
+    total_ret = test_fail;
     if (0 != (rc = server_barrier())) {
         total_ret++;
         goto exit;
@@ -852,11 +979,6 @@ int server_finalize(test_params *params)
         PMIX_LIST_RELEASE(server_list);
         TEST_VERBOSE(("SERVER %d FINALIZE PID:%d with status %d",
                     my_server_id, getpid(), ret));
-        if (0 == total_ret) {
-            TEST_OUTPUT(("Test finished OK!"));
-        } else {
-            rc = PMIX_ERROR;
-        }
     }
     PMIX_LIST_RELEASE(server_nspace);
 
@@ -866,6 +988,11 @@ int server_finalize(test_params *params)
         total_ret += rc;
         goto exit;
     }
+    if (0 == total_ret) {
+        TEST_OUTPUT(("Test finished OK!"));
+    } else {
+        TEST_OUTPUT(("Test FAILED!"));
+    }
 
 exit:
     return total_ret;
@@ -932,6 +1059,7 @@ int server_launch_clients(int local_size, int univ_size, int base_rank,
             cli_kill_all();
             return rc;
         }
+        TEST_VERBOSE(("run %s:%d", proc.nspace, proc.rank));
 
         cli_info[cli_counter].pid = fork();
         if (cli_info[cli_counter].pid < 0) {
@@ -966,6 +1094,15 @@ int server_launch_clients(int local_size, int univ_size, int base_rank,
         pmix_argv_append_nosize(&client_argv, digit);
 
         if (cli_info[cli_counter].pid == 0) {
+            sigset_t sigs;
+            set_handler_default(SIGTERM);
+            set_handler_default(SIGINT);
+            set_handler_default(SIGHUP);
+            set_handler_default(SIGPIPE);
+            set_handler_default(SIGCHLD);
+            sigprocmask(0, 0, &sigs);
+            sigprocmask(SIG_UNBLOCK, &sigs, 0);
+
             if( !TEST_VERBOSE_GET() ){
                 // Hide clients stdout
                 if (NULL == freopen("/dev/null","w", stdout)) {
@@ -977,6 +1114,7 @@ int server_launch_clients(int local_size, int univ_size, int base_rank,
             TEST_ERROR(("execve() failed"));
             return 0;
         }
+        cli_info[cli_counter].alive = true;
         cli_info[cli_counter].state = CLI_FORKED;
 
         pmix_argv_free(client_argv);
